@@ -1,6 +1,6 @@
 #  MIT License
 #
-#  Copyright (c) 2017-2023 TileDB Inc.
+#  Copyright (c) 2017-2024 TileDB Inc.
 #
 #  Permission is hereby granted, free of charge, to any person obtaining a copy
 #  of this software and associated documentation files (the "Software"), to deal
@@ -63,7 +63,7 @@ tiledb_vfs <- function(config = NULL, ctx = tiledb_get_context()) {
 #' @return The uri value
 #' @export
 tiledb_vfs_create_bucket <- function(uri, vfs = tiledb_get_vfs()) {
-  stopifnot(`Argument 'vfs' must a tiledb_vfs object` = is(vfs, "tiledb_vfs"),
+  stopifnot(`Argument 'vfs' must be a tiledb_vfs object` = is(vfs, "tiledb_vfs"),
             `Argument 'uri' must be character` = is.character(uri))
   libtiledb_vfs_create_bucket(vfs@ptr, uri)
 }
@@ -75,7 +75,7 @@ tiledb_vfs_create_bucket <- function(uri, vfs = tiledb_get_vfs()) {
 #' @return The uri value
 #' @export
 tiledb_vfs_remove_bucket <- function(uri, vfs = tiledb_get_vfs()) {
-  stopifnot(`Argument 'vfs' must a tiledb_vfs object` = is(vfs, "tiledb_vfs"),
+  stopifnot(`Argument 'vfs' must be a tiledb_vfs object` = is(vfs, "tiledb_vfs"),
             `Argument 'uri' must be character` = is.character(uri))
   libtiledb_vfs_remove_bucket(vfs@ptr, uri)
 }
@@ -96,7 +96,7 @@ tiledb_vfs_remove_bucket <- function(uri, vfs = tiledb_get_vfs()) {
 #' tiledb_vfs_is_bucket(vfs, "s3://tiledb-public-us-west-1/test-array-4x4")
 #' }
 tiledb_vfs_is_bucket <- function(uri, vfs = tiledb_get_vfs()) {
-  stopifnot(`Argument 'vfs' must a tiledb_vfs object` = is(vfs, "tiledb_vfs"),
+  stopifnot(`Argument 'vfs' must be a tiledb_vfs object` = is(vfs, "tiledb_vfs"),
             `Argument 'uri' must be character` = is.character(uri))
   libtiledb_vfs_is_bucket(vfs@ptr, uri)
 }
@@ -117,7 +117,7 @@ tiledb_vfs_is_bucket <- function(uri, vfs = tiledb_get_vfs()) {
 #' tiledb_vfs_is_empty_bucket(vfs, "s3://tiledb-public-us-west-1/test-array-4x4")
 #' }
 tiledb_vfs_is_empty_bucket <- function(uri, vfs = tiledb_get_vfs()) {
-  stopifnot(`Argument 'vfs' must a tiledb_vfs object` = is(vfs, "tiledb_vfs"),
+  stopifnot(`Argument 'vfs' must be a tiledb_vfs object` = is(vfs, "tiledb_vfs"),
             `Argument 'uri' must be character` = is.character(uri))
   libtiledb_vfs_is_empty_bucket(vfs@ptr, uri)
 }
@@ -129,7 +129,7 @@ tiledb_vfs_is_empty_bucket <- function(uri, vfs = tiledb_get_vfs()) {
 #' @return The URI value that was emptied
 #' @export
 tiledb_vfs_empty_bucket <- function(uri, vfs = tiledb_get_vfs()) {
-  stopifnot(`Argument 'vfs' must a tiledb_vfs object` = is(vfs, "tiledb_vfs"),
+  stopifnot(`Argument 'vfs' must be a tiledb_vfs object` = is(vfs, "tiledb_vfs"),
             `Argument 'uri' must be character` = is.character(uri))
   libtiledb_vfs_empty_bucket(vfs@ptr, uri)
 }
@@ -141,7 +141,7 @@ tiledb_vfs_empty_bucket <- function(uri, vfs = tiledb_get_vfs()) {
 #' @return The uri value of the created directory
 #' @export
 tiledb_vfs_create_dir <- function(uri, vfs = tiledb_get_vfs()) {
-  stopifnot(`Argument 'vfs' must a tiledb_vfs object` = is(vfs, "tiledb_vfs"),
+  stopifnot(`Argument 'vfs' must be a tiledb_vfs object` = is(vfs, "tiledb_vfs"),
             `Argument 'uri' must be character` = is.character(uri))
   libtiledb_vfs_create_dir(vfs@ptr, uri)
 }
@@ -153,7 +153,7 @@ tiledb_vfs_create_dir <- function(uri, vfs = tiledb_get_vfs()) {
 #' @return A boolean value indicating if it is a directory
 #' @export
 tiledb_vfs_is_dir <- function(uri, vfs = tiledb_get_vfs()) {
-  stopifnot(`Argument 'vfs' must a tiledb_vfs object` = is(vfs, "tiledb_vfs"),
+  stopifnot(`Argument 'vfs' must be a tiledb_vfs object` = is(vfs, "tiledb_vfs"),
             `Argument 'uri' must be character` = is.character(uri))
   libtiledb_vfs_is_dir(vfs@ptr, uri)
 }
@@ -165,7 +165,7 @@ tiledb_vfs_is_dir <- function(uri, vfs = tiledb_get_vfs()) {
 #' @return The uri value of the removed directory
 #' @export
 tiledb_vfs_remove_dir <- function(uri, vfs = tiledb_get_vfs()) {
-  stopifnot(`Argument 'vfs' must a tiledb_vfs object` = is(vfs, "tiledb_vfs"),
+  stopifnot(`Argument 'vfs' must be a tiledb_vfs object` = is(vfs, "tiledb_vfs"),
             `Argument 'uri' must be character` = is.character(uri))
   invisible(libtiledb_vfs_remove_dir(vfs@ptr, uri))
 }
@@ -177,7 +177,7 @@ tiledb_vfs_remove_dir <- function(uri, vfs = tiledb_get_vfs()) {
 #' @return A boolean value indicating if it is a file
 #' @export
 tiledb_vfs_is_file <- function(uri, vfs = tiledb_get_vfs()) {
-  stopifnot(`Argument 'vfs' must a tiledb_vfs object` = is(vfs, "tiledb_vfs"),
+  stopifnot(`Argument 'vfs' must be a tiledb_vfs object` = is(vfs, "tiledb_vfs"),
             `Argument 'uri' must be character` = is.character(uri))
   libtiledb_vfs_is_file(vfs@ptr, uri)
 }
@@ -189,7 +189,7 @@ tiledb_vfs_is_file <- function(uri, vfs = tiledb_get_vfs()) {
 #' @return The uri value of the removed file
 #' @export
 tiledb_vfs_remove_file <- function(uri, vfs = tiledb_get_vfs()) {
-  stopifnot(`Argument 'vfs' must a tiledb_vfs object` = is(vfs, "tiledb_vfs"),
+  stopifnot(`Argument 'vfs' must be a tiledb_vfs object` = is(vfs, "tiledb_vfs"),
             `Argument 'uri' must be character` = is.character(uri))
   libtiledb_vfs_remove_file(vfs@ptr, uri)
 }
@@ -201,7 +201,7 @@ tiledb_vfs_remove_file <- function(uri, vfs = tiledb_get_vfs()) {
 #' @return The size of the file
 #' @export
 tiledb_vfs_file_size <- function(uri, vfs = tiledb_get_vfs()) {
-  stopifnot(`Argument 'vfs' must a tiledb_vfs object` = is(vfs, "tiledb_vfs"),
+  stopifnot(`Argument 'vfs' must be a tiledb_vfs object` = is(vfs, "tiledb_vfs"),
             `Argument 'uri' must be character` = is.character(uri))
   libtiledb_vfs_file_size(vfs@ptr, uri)
 }
@@ -214,7 +214,7 @@ tiledb_vfs_file_size <- function(uri, vfs = tiledb_get_vfs()) {
 #' @return The newuri value of the moved file
 #' @export
 tiledb_vfs_move_file <- function(olduri, newuri, vfs = tiledb_get_vfs()) {
-  stopifnot(`Argument 'vfs' must a tiledb_vfs object` = is(vfs, "tiledb_vfs"),
+  stopifnot(`Argument 'vfs' must be a tiledb_vfs object` = is(vfs, "tiledb_vfs"),
             `Argument 'olduri' must be character` = is.character(olduri),
             `Argument 'newuri' must be character` = is.character(newuri))
   libtiledb_vfs_move_file(vfs@ptr, olduri, newuri)
@@ -228,7 +228,7 @@ tiledb_vfs_move_file <- function(olduri, newuri, vfs = tiledb_get_vfs()) {
 #' @return The newuri value of the moved directory
 #' @export
 tiledb_vfs_move_dir <- function(olduri, newuri, vfs = tiledb_get_vfs()) {
-  stopifnot(`Argument 'vfs' must a tiledb_vfs object` = is(vfs, "tiledb_vfs"),
+  stopifnot(`Argument 'vfs' must be a tiledb_vfs object` = is(vfs, "tiledb_vfs"),
             `Argument 'olduri' must be character` = is.character(olduri),
             `Argument 'newuri' must be character` = is.character(newuri))
   libtiledb_vfs_move_dir(vfs@ptr, olduri, newuri)
@@ -241,7 +241,7 @@ tiledb_vfs_move_dir <- function(olduri, newuri, vfs = tiledb_get_vfs()) {
 #' @return The uri value
 #' @export
 tiledb_vfs_touch <- function(uri, vfs = tiledb_get_vfs()) {
-  stopifnot(`Argument 'vfs' must a tiledb_vfs object` = is(vfs, "tiledb_vfs"),
+  stopifnot(`Argument 'vfs' must be a tiledb_vfs object` = is(vfs, "tiledb_vfs"),
             `Argument 'uri' must be character` = is.character(uri))
   libtiledb_vfs_touch(vfs@ptr, uri)
 }
@@ -269,7 +269,7 @@ tiledb_get_vfs <- function() {
 #' storing the VFS object.
 #' @export
 tiledb_set_vfs <- function(vfs) {
-  stopifnot(`Argument 'vfs' must a tiledb_vfs object` = is(vfs, "tiledb_vfs"))
+  stopifnot(`Argument 'vfs' must be a tiledb_vfs object` = is(vfs, "tiledb_vfs"))
   ## set the ctx entry from the package environment (a lightweight hash)
   .pkgenv[["vfs"]] <- vfs
   invisible(NULL)
@@ -287,8 +287,8 @@ tiledb_set_vfs <- function(vfs) {
 tiledb_vfs_open <- function(binfile, mode = c("READ", "WRITE", "APPEND"),
                             vfs = tiledb_get_vfs(), ctx = tiledb_get_context()) {
   mode <- match.arg(mode)
-  stopifnot(`Argument 'vfs' must a tiledb_vfs object` = is(vfs, "tiledb_vfs"),
-            `Argument 'ctx' must a tiledb_ctx object` = is(ctx, "tiledb_ctx"),
+  stopifnot(`Argument 'vfs' must be a tiledb_vfs object` = is(vfs, "tiledb_vfs"),
+            `Argument 'ctx' must be a tiledb_ctx object` = is(ctx, "tiledb_ctx"),
             `Argument 'binfile' must be character` = is.character(binfile))
   libtiledb_vfs_open(ctx@ptr, vfs@ptr, binfile, mode)
 }
@@ -301,7 +301,7 @@ tiledb_vfs_open <- function(binfile, mode = c("READ", "WRITE", "APPEND"),
 #' @export
 tiledb_vfs_close <- function(fh, ctx = tiledb_get_context()) {
   stopifnot(`Argument 'fh' must be an external pointer` = is(fh, "externalptr"),
-            `Argument 'ctx' must a tiledb_ctx object` = is(ctx, "tiledb_ctx"))
+            `Argument 'ctx' must be a tiledb_ctx object` = is(ctx, "tiledb_ctx"))
   libtiledb_vfs_close(ctx@ptr, fh)
 }
 
@@ -313,7 +313,7 @@ tiledb_vfs_close <- function(fh, ctx = tiledb_get_context()) {
 #' @export
 tiledb_vfs_sync <- function(fh, ctx = tiledb_get_context()) {
   stopifnot(`Argument 'fh' must be an external pointer` = is(fh, "externalptr"),
-            `Argument 'ctx' must a tiledb_ctx object` = is(ctx, "tiledb_ctx"))
+            `Argument 'ctx' must be a tiledb_ctx object` = is(ctx, "tiledb_ctx"))
   libtiledb_vfs_sync(ctx@ptr, fh)
 }
 
@@ -331,7 +331,7 @@ tiledb_vfs_sync <- function(fh, ctx = tiledb_get_context()) {
 tiledb_vfs_write <- function(fh, vec, ctx = tiledb_get_context()) {
   stopifnot(`Argument 'fh' must be an external pointer` = is(fh, "externalptr"),
             `Argument 'vec' must be integer` = is.integer(vec),
-            `Argument 'ctx' must a tiledb_ctx object` = is(ctx, "tiledb_ctx"))
+            `Argument 'ctx' must be a tiledb_ctx object` = is(ctx, "tiledb_ctx"))
   libtiledb_vfs_write(ctx@ptr, fh, vec)
 }
 
@@ -356,7 +356,7 @@ tiledb_vfs_read <- function(fh, offset, nbytes, ctx = tiledb_get_context()) {
               "Argument 'offset' must be integer64" = is(offset, "integer64"),
               "Argument 'nbytes' currently a required parameter" = !missing(nbytes),
               "Argument 'nbytes' must be integer64" = is(nbytes, "integer64"),
-              "Argument 'ctx' must a tiledb_ctx object" = is(ctx, "tiledb_ctx"))
+              "Argument 'ctx' must be a tiledb_ctx object" = is(ctx, "tiledb_ctx"))
     libtiledb_vfs_read(ctx@ptr, fh, offset, nbytes)
 }
 
@@ -367,7 +367,7 @@ tiledb_vfs_read <- function(fh, offset, nbytes, ctx = tiledb_get_context()) {
 #' @return The size of the directory
 #' @export
 tiledb_vfs_dir_size <- function(uri, vfs = tiledb_get_vfs()) {
-  stopifnot(`Argument 'vfs' must a tiledb_vfs object` = is(vfs, "tiledb_vfs"),
+  stopifnot(`Argument 'vfs' must be a tiledb_vfs object` = is(vfs, "tiledb_vfs"),
             `Argument 'uri' must be character` = is.character(uri))
   libtiledb_vfs_dir_size(vfs@ptr, uri)
 }
@@ -379,7 +379,7 @@ tiledb_vfs_dir_size <- function(uri, vfs = tiledb_get_vfs()) {
 #' @return The content of the directory, non-recursive
 #' @export
 tiledb_vfs_ls <- function(uri, vfs = tiledb_get_vfs()) {
-  stopifnot(`Argument 'vfs' must a tiledb_vfs object` = is(vfs, "tiledb_vfs"),
+  stopifnot(`Argument 'vfs' must be a tiledb_vfs object` = is(vfs, "tiledb_vfs"),
             `Argument 'uri' must be character` = is.character(uri))
   libtiledb_vfs_ls(vfs@ptr, uri)
 }
@@ -391,7 +391,7 @@ tiledb_vfs_ls <- function(uri, vfs = tiledb_get_vfs()) {
 #' @return The unserialized object
 #' @export
 tiledb_vfs_unserialize <- function(uri, vfs = tiledb_get_vfs()) {
-    stopifnot("Argument 'vfs' must a tiledb_vfs object" = is(vfs, "tiledb_vfs"),
+    stopifnot("Argument 'vfs' must be a tiledb_vfs object" = is(vfs, "tiledb_vfs"),
               "Argument 'uri' must be character" = is.character(uri))
     n <- tiledb_vfs_file_size(uri)
     fh <- tiledb_vfs_open(uri, "READ")
@@ -413,7 +413,7 @@ tiledb_vfs_unserialize <- function(uri, vfs = tiledb_get_vfs()) {
 #' @return The uri is returned invisibly
 #' @export
 tiledb_vfs_serialize <- function(obj, uri, vfs = tiledb_get_vfs()) {
-    stopifnot("Argument 'vfs' must a tiledb_vfs object" = is(vfs, "tiledb_vfs"),
+    stopifnot("Argument 'vfs' must be a tiledb_vfs object" = is(vfs, "tiledb_vfs"),
               "Argument 'uri' must be character" = is.character(uri))
     ## We could not find a direct conversion of the 'raw' vector we get from serialize()
     ## into a format that corresponded to what saveRDS() writes so we cheat a little
@@ -445,9 +445,27 @@ tiledb_vfs_serialize <- function(obj, uri, vfs = tiledb_get_vfs()) {
 #' @return The uri value of the removed file
 #' @export
 tiledb_vfs_copy_file <- function(file, uri, vfs = tiledb_get_vfs()) {
-    stopifnot("Argument 'vfs' must a tiledb_vfs object" = is(vfs, "tiledb_vfs"),
+    stopifnot("Argument 'vfs' must be a tiledb_vfs object" = is(vfs, "tiledb_vfs"),
               "Argument 'uri' must be character" = is.character(uri),
               "Argument 'file' must be character and point to a file" =
                   is.character(uri) && file.exists(file))
     libtiledb_vfs_copy_file(vfs@ptr, file, uri)
 }
+
+#' Recursively list objects from given URI
+#'
+#' This functionality is currently limited to S3 URIs.
+#'
+#' @param uri Character variable with a URI describing a file path
+#' @param vfs (optiona) A TileDB VFS object; default is to use a cached value.
+#' @param ctx (optional) A TileDB Ctx object
+#' @return A data.frame object with two columns for the full path and the object
+#' size in bytes
+#' @export
+tiledb_vfs_ls_recursive <- function(uri, vfs = tiledb_get_vfs(), ctx = tiledb_get_context()) {
+    stopifnot("Argument 'vfs' must be a tiledb_vfs object" = is(vfs, "tiledb_vfs"),
+              "Argument 'ctx' must be a tiledb_ctx object" = is(ctx, "tiledb_ctx"),
+              "Argument 'uri' must be character variable" = is.character(uri),
+              "This function needs TileDB 2.22.0 or later" = tiledb_version(TRUE) >= "2.22.0")
+    libtiledb_vfs_ls_recursive(ctx@ptr, vfs@ptr, uri)
+}
